"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_tag_1 = __importDefault(require("graphql-tag"));
var apollo_link_1 = require("apollo-link");
var getGroup = function (collapsed) {
    return collapsed ? console.groupCollapsed.bind(console) : console.group.bind(console);
};
var parseQuery = function (queryString) {
    var queryObj = graphql_tag_1.default(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n    ", "\n  "], ["\n    ", "\n  "])), queryString);
    var name = queryObj.definitions[0].name;
    return [name ? name.value : 'Generic', queryString.trim()];
};
var logObject = function (obj, name, collapsed) {
    if (collapsed === void 0) { collapsed = false; }
    getGroup(collapsed)(name);
    console.log(JSON.stringify(obj, null, '  '));
    console.groupEnd();
};
exports.loggerLink = function (getSchemaName) {
    return new apollo_link_1.ApolloLink(function (operation, forward) {
        var schemaName = getSchemaName(operation);
        operation.setContext({ start: Date.now() });
        var variables = operation.variables;
        var operationType = operation.query.definitions[0].operation;
        var _a = parseQuery(operation.query.loc.source.body), queryName = _a[0], query = _a[1];
        if (operationType === 'subscription') {
            var date = new Date().toLocaleTimeString();
            console.groupCollapsed("%cgraphql %c" + operationType + " %c" + schemaName + "::%c" + queryName + " %c@ " + date, "%c" + operationType + " %c" + schemaName + "::%c" + queryName + " %c@ " + date + ")", "color: #61A600; font-weight: bold;", 'color: gray; font-weight: lighter', 'color: black; font-weight: bold', 'color: gray; font-weight: lighter');
            if (variables) {
                Object.keys(variables).forEach(function (key) {
                    console.log("%c" + key + ": %c" + variables[key], 'color: gray; font-weight: lighter', 'color: black; font-weight: bold');
                });
            }
            console.groupCollapsed('QUERY');
            console.log(query);
            console.groupEnd();
            console.groupEnd();
            return forward(operation);
        }
        return forward(operation).map(function (result) {
            var _a, _b, _c;
            var time = Date.now() - operation.getContext().start;
            var errors = (_a = result.errors) !== null && _a !== void 0 ? _a : (_c = (_b = result.data) === null || _b === void 0 ? void 0 : _b[queryName]) === null || _c === void 0 ? void 0 : _c.errors;
            var hasError = Boolean(errors);
            try {
                getGroup(!hasError)("%c" + operationType + " %c" + schemaName + "::%c" + queryName + " %c(in " + time + " ms)", "color: " + (hasError ? '#F51818' : '#61A600') + "; font-weight: bold;", 'color: gray; font-weight: lighter', 'color: black; font-weight: bold', 'color: gray; font-weight: lighter');
                if (errors) {
                    errors.forEach(function (err) {
                        console.log("%c" + err.message, 'color: #F51818; font-weight: lighter');
                    });
                }
                if (variables && Object.keys(variables).length !== 0) {
                    logObject(variables, 'VARIABLES', true);
                }
                console.groupCollapsed('QUERY');
                console.log(query);
                console.groupEnd();
                if (result.data)
                    logObject(result.data, 'RESULT', true);
                if (errors)
                    logObject(errors, 'ERRORS', true);
                console.groupEnd();
            }
            catch (_d) {
                // this may happen if console group is not supported
                console.log(operationType + " " + schemaName + "::" + queryName + " (in " + time + " ms)");
                if (errors)
                    console.error(errors);
            }
            return result;
        });
    });
};
var templateObject_1;
